@@ -15,7 +15,7 @@
 	   atomCodes/2, atEndOfStream/1,
 	   isMod/3, isRem/3,
 	   unifyWithOccursCheck/2,
-           readLine/1,
+           readLineOn/0, readLine/1,
 	   waitConcurrentConjunction/6,
 	   appendAtom/3,
 	   map1M/2, map2M/3, map1partialM/2, map2partialM/3,
@@ -33,6 +33,7 @@
 	   fileExistsAndNewer/2, canWriteFile/1, currentPID/1, sleepSeconds/1,
 	   getHostname/1, shellCmd/1, shellCmd/2,
 	   execCommand/4, forkProcessForGoal/1,
+           stdInputStream/1, stdOutputStream/1, stdErrorStream/1,
 	   isInputStream/1, isOutputStream/1, isTerminalDeviceStream/1,
 	   currentClockTime/1, clocktime2localtime/8, clocktime2utctime/7,
 	   date2clocktime/8,
@@ -166,7 +167,7 @@ isRem(X,Y,Z) :- X is rem(Y,Z).
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % get Sicstus version (e.g., '3.9'):
-% Currently, we support only 3.5, 3.7, 3.8, 3.9, 3.10, 3.11, 3.12, 4.0, 4.1, 4.2:
+% Currently, we support only a restricted set of versions:
 getSicstusVersion(SV) :-
 	prolog_flag(version,V,V),
 	atom_chars(V,Vs),
@@ -179,11 +180,7 @@ getSicstusVersion(SV) :-
 	 app("SICStus 3.10",_,Vs) -> SV='3.10' ;
 	 app("SICStus 3.11",_,Vs) -> SV='3.11' ;
 	 app("SICStus 3.12",_,Vs) -> SV='3.12' ;
-	 app("SICStus 4.0",_,Vs)  -> SV='4.0' ;
-	 app("SICStus 4.1",_,Vs)  -> SV='4.1' ;
-	 app("SICStus 4.2",_,Vs)  -> SV='4.2' ;
-	 app("SICStus 4.3",_,Vs)  -> SV='4.3' ;
-	 app("SICStus 4.4",_,Vs)  -> SV='4.4' ;
+	 app("SICStus 4.",_,Vs)  -> SV='4.x' ;
 	 write(user_error,'ERROR: UNKNOWN SICSTUS PROLOG VERSION:'),
 	 nl(user_error),
 	 write(user_error,'PLEASE MODIFY pakcs/src/sicstusbasics.pl'),
@@ -247,6 +244,9 @@ generatePrologBasics :-	sicstus38orHigher, !,
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+% activate GNU readline behavior (not available for SICStus-Prolog)
+readLineOn :- true.
+
 % read a single line from stdin (return end_of_file if there is no more input)
 readLine(Input) :-
 	get_code(C),
@@ -511,6 +511,18 @@ forkProcessForGoal(Goal) :-
 	shellCmd(ForkCmd).
 
 
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+% Stream-related operations.
+
+% the standard input stream
+stdInputStream(Stream) :- prolog_flag(user_input,Stream).
+
+% the standard output stream
+stdOutputStream(Stream) :- prolog_flag(user_output,Stream).
+
+% the standard error stream
+stdErrorStream(Stream) :- prolog_flag(user_error,Stream).
+
 % is a stream a readable stream?
 isInputStream(Stream) :-
 	sicstus38orHigher -> stream_property(Stream,input)
